#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>
#include <cassert>

using namespace std;

static const int kModulo = 1000 * 1000 * 1000 + 7;

int pow(int x, int y) {
    if (y == 0)
        return 1;
    if (y % 2)
        return 1LL * pow(x, y - 1) * x % kModulo;

    int v = pow(x, y / 2);
    return 1LL * v * v % kModulo;
}

int position(const vector<int>& V, int SIGMA) {
    if (SIGMA == 1)
        return 0;

    int split = -1;
    for (int i = 1; i < int(V.size()); ++i) {
        int diff = V[i] - V[i - 1];
        if (diff < 0)
            diff += SIGMA;
        if (diff != 1) {
            split = i;
            break;
        }
    }

    while (split > SIGMA)
        split -= SIGMA;
    if (int(V.size()) > 2 * SIGMA && split == -1) // no split and it's too long
        return -1;

    if (split == -1) { // it's pretty short and no split yet, easy to do
        if (int(V.size()) <= SIGMA) {
            int min_val = max(0, V[0] - (SIGMA - int(V.size())));
            return (1LL * min_val * SIGMA + V[0] - min_val) % kModulo;
        }

        int min_val = max(1, V[0] - (2 * SIGMA - int(V.size())));
        if (V[0] == 0)
            min_val = (SIGMA - (2 * SIGMA - int(V.size()))) % (SIGMA);

        int added_in_front = V[0] - min_val;
        if (added_in_front < 0)
            added_in_front += SIGMA;

        vector<int> aux(2, min_val);
        int answer = position(aux, SIGMA);
        answer = (1LL * answer * SIGMA + added_in_front) % kModulo;
        if (answer < 0)
            answer += kModulo;
        return answer;
    }

    // we have a split, it might be for only 2 characters
    if (V.size() == 2) {
        int diff = V[1] - V[0];
        if (diff <= 0)
            diff += SIGMA;

        // SIGMA-diff 0 would be at positions -1 0
        if (V[1] > 0) { // this is easy
            int where = (1LL * pow(SIGMA, diff - 1) * V[1] - 1) % kModulo;
            if (where < 0)
                where += kModulo;
            return where;
        } else { // we need to go the whole way around, all the way to the back
            int where = (2 * pow(SIGMA, diff) - pow(SIGMA, diff - 1) - 1) % kModulo;
            if (where < 0)
                where += kModulo;
            return where;
        }
    }

    // now we can reduce, yey
    // check it's ok
    for (int i = 1; i < int(V.size()); ++i) {
        bool is_split = ((V[i] - V[i - 1] + SIGMA) % SIGMA != 1);
        if (is_split && (i % SIGMA != split % SIGMA))
            return -1; // split needs to be every SIGMA
    }

    // the rewrite is easy, yeeey
    vector<int> aux;
    aux.push_back((V[split - 1] + 1) % SIGMA);
    for (int i = split; i < int(V.size()); i += SIGMA)
        aux.push_back(V[i]);
    int answer = position(aux, SIGMA);
    if (answer < 0)
        return answer;
    return (1LL * answer * SIGMA + SIGMA - split) % kModulo;
}

int main() {
    ifstream cin("pscfft.in");
    ofstream cout("pscfft.out");

    int T; assert(cin >> T);
    assert(1 <= T && T <= 500 * 1000);

    int sumN = 0;
    for (int i = 0; i < T; ++i) {
        int N, SIGMA; assert(cin >> N >> SIGMA);
        assert(1 <= N && N <= 500 * 1000);
        assert(1 <= SIGMA && SIGMA <= 1000 * 1000 * 1000);
        sumN += N;
        assert(sumN <= 500 * 1000);

        vector<int> V(N);
        for (int i = 0; i < N; ++i) {
            assert(cin >> V[i]);
            assert(0 <= V[i] && V[i] < SIGMA);
        }

        cout << position(V, SIGMA) << "\n";
    }
}
