/*
	Eric Petru Stavarache
	Problema countfefete - Proba de baraj 2018
	Solutie O( N^2 * log VALMAX )
*/

#include <bits/stdc++.h>

using namespace std;

const int kMod = 1000000000 + 7;

int mul(int x, int y)
{
    return static_cast<uint64_t>(x) * static_cast<uint64_t>(y) % kMod;
}

template<typename... Args>
int mul(int x, Args... args)
{
    return static_cast<uint64_t>(x) * static_cast<uint64_t>(mul(args...)) % kMod;
}

int add(int x, int y)
{
    int ret = x + y;
    if (ret >= kMod) {
        ret -= kMod;
    }
    if (ret < 0) {
        ret += kMod;
    }
    return ret;
}

template<typename... Args>
int add(int x, Args... args)
{
    int ret = x + add(args...);
    if (ret >= kMod) {
        ret -= kMod;
    }
    if (ret < 0) {
        ret += kMod;
    }
    return ret;
}

int pow(int b, int p)
{
    int ret = 1;
    int at = b;
    while (p > 0) {
        if (p % 2 == 1) {
            ret = mul(ret, at);
        }
        at = mul(at, at);

        p /= 2;
    }
    return ret;
}

// Computes 2^p mod kMod.
int pow2(int p)
{
    return pow(2, p);
}

struct countfefete
{
    int n;
    vector<int> v;
    vector<vector<int>> G;
    vector<bool> viz;
    // DP[node][bit] = cate noduri exista in subarborele lui `node`, care sa aiba
    // bitul iterat curent setat sa fie `bit`.
    vector<array<int, 2>> DP;
    vector<int> fact;
    vector<int> fact_inv;


    countfefete(int n, vector<int> v, vector<vector<int>> G):
        n(n),
        v(v),
        G(move(G)),
        viz(n),
        DP(n),
        fact(n + 1),
        fact_inv(n + 1)
    {
        fact[0] = 1;
        for (int i = 1; i <= n; ++i) {
            fact[i] = mul(fact[i - 1], i);
        }
        for (int i = 0; i <= n; ++i) {
            fact_inv[i] = inv(fact[i]);
        }
    }

    int inv(int x)
    {
        return pow(x, kMod - 2);
    }

    int choose(int n, int k)
    {
        if (k > n) {
            return 0;
        }
        return mul(fact[n], fact_inv[k], fact_inv[n - k]);
    }


    bool isset(int node, int bit)
    {
        return (v[node] & (1 << bit)) != 0;
    }

    void DFS(int node, int bit, int orig)
    {
        viz[node] = true;
        for (auto son : G[node])
        {
            if (viz[son]) {
                continue;
            }

            // In case there are nodes with equal values, the one with the lowest index takes the cake.
            if (v[son] < v[orig] ||
                    (v[son] == v[orig] && son < orig)
               ) {
                continue;
            }

            DFS(son, bit, orig);
            for (int b = 0; b <= 1; ++b) {
                DP[node][b] += DP[son][b];
            }
        }
        DP[node][isset(node, bit)]++;
    }

    // Return in how many ways we can choose a non-empty subset from DP,
    // such that their xor will respect `set`.
    int choices_for_node(array<int, 2> DP, bool set)
    {
        int common = pow2(DP[0]);
        int sum = 0;

        if (set) {
            // We have to choose an even number of ones, and any number of zeroes.
            // When i == 0, we have to choose a non-empty amount of zeroes.
            for (int i = 2; i <= DP[1]; i += 2) {
                // Choose i ones out of DP[node][1].
                sum = add(sum, choose(DP[1], i));
            }
            return add(pow2(DP[0]) - 1, mul(sum, common));
            // sum * common times we will include bit `bit`.
        }
        // Bit `bit` is not set. If we want to include it, we have to choose an odd number of ones, and any number of zeroes.
        for (int i = 1; i <= DP[1]; i += 2) {
            // Choose i ones out of DP[1].
            sum = add(sum, choose(DP[1], i));
        }
        return mul(sum, common);
    };

    int answer_for_node(int node, int bit)
    {
        fill(viz.begin(), viz.end(), false);
        fill(DP.begin(), DP.end(), array<int, 2>{0, 0});

        DFS(node, bit, node);
        bool set = isset(node, bit);

        int answer = choices_for_node(DP[node], set);
        for (auto son : G[node]) {
            answer = add(answer, -choices_for_node(DP[son], set));
        }
        return mul((1 << bit) % kMod, answer);
    }

    int solve()
    {
        int ans = 0;
        for (int b = 0; b < 31; ++b) {
            for (int i = 0; i < n; ++i) {
                ans += answer_for_node(i, b);
                if (ans >= kMod) {
                    ans -= kMod;
                }
            }
        }
        return ans;
    }
};

int main()
{
    ifstream fin("countfefete.in");
    ofstream fout("countfefete.out");

    int n;
    fin >> n;

    vector<int> v(n);
    for (int i = 0; i < n; ++i) {
        fin >> v[i];
    }

    vector<vector<int>> G(n);
    for (int i = 1; i < n; ++i) {
        int x, y;
        fin >> x >> y;
        --x;
        --y;
        G[x].push_back(y);
        G[y].push_back(x);
    }

    countfefete cfft(n, v, G);
    fout << cfft.solve() << "\n";

    return 0;
}
