/* 
	Ioan Bogdan Iordache
	Problema countfefete - Proba de baraj 2018
	Solutie O( N * log N * log VALMAX )
*/

#include <bits/stdc++.h>
using namespace std;

const int kMod = 1000000007;

struct Node {
	vector< int > cnt0, cnt1;
	Node() : cnt0(31), cnt1(31) {};
	void combine(const Node& rhs) {
		for (int bit = 0; bit < 31; ++bit) {
			cnt0[bit] += rhs.cnt0[bit];
			cnt1[bit] += rhs.cnt1[bit];
		}
	}
};

struct DisjointSet {
	vector< int > parent, weight;
	vector< bool > active;
	vector< Node > data;

	DisjointSet(int n = 0) : parent(n), weight(n, 1), active(n, false), data(n) {};

	int get_root(int node) {
		int aux = node;
		while (parent[node])
			node = parent[node];
		int root = node;
		node = aux;
		while (node != root) {
			aux = parent[node];
			parent[node] = root;
			node = aux;
		}
		return root;
	}

	void join(int x, int y) {
		x = get_root(x);
		y = get_root(y);
		if (weight[x] > weight[y]) {
			parent[y] = x;
			weight[x] += weight[y];
			data[x].combine(data[y]);
		}
		else {
			parent[x] = y;
			weight[y] += weight[x];
			data[y].combine(data[x]);
		}
	}
};

int main() {
	ifstream cin("countfefete.in");
	ofstream cout("countfefete.out");

	int n; cin >> n;
	vector< int > vals(n + 1);
	for (int i = 1; i <= n; ++i)
		cin >> vals[i];
	vector< vector<int> > tree(n + 1, vector<int>(0));
	for (int i = 1; i < n; ++i) {
		int x, y; cin >> x >> y;
		tree[x].push_back(y);
		tree[y].push_back(x);
	}

	vector< int > nodes(n + 1);
	for (int i = 1; i <= n; ++i)
		nodes[i] = i;
	sort(nodes.begin() + 1, nodes.end(), [&](int a, int b) {
		return vals[a] < vals[b];
	});

	vector< int > pw2(max(35, 5 * n + 5));
	pw2[0] = 1;
	for (int i = 1; i < max(5 * n + 5, 35); ++i)
		pw2[i] = (pw2[i-1] + pw2[i-1]) % kMod;

	DisjointSet dsu(n + 1);
	for (int i = 1; i <= n; ++i) {
		for (int bit = 0; bit < 31; ++bit)
			if ((1 << bit) & vals[i])
				dsu.data[i].cnt1[bit] = 1;
			else
				dsu.data[i].cnt0[bit] = 1;
	}
	int sol = 0;
	for (int i = n; i; --i) {
		int node = nodes[i];

		dsu.active[node] = true;
		vector<int> cnt0(dsu.data[node].cnt0), cnt1(dsu.data[node].cnt1);
		for (int nxt : tree[node]) {
			if (!dsu.active[nxt])
				continue;

			int adj = dsu.get_root(nxt);
			for (int bit = 0; bit < 31; ++bit) {
				cnt0[bit] += dsu.data[adj].cnt0[bit];
				cnt1[bit] += dsu.data[adj].cnt1[bit];
			}
		}

		
		for (int bit = 0; bit < 31; ++bit) {
			if (vals[node] & (1 << bit)) {
				if (cnt1[bit] > 0)
					sol = int( (sol + 1LL * (pw2[cnt0[bit] + cnt1[bit] - 1] - 1 + kMod) * pw2[bit]) % kMod );
				else
					sol = (sol + 1LL * (pw2[cnt0[bit]] - 1 + kMod) * pw2[bit]) % kMod;
			}
			else {
				if (cnt1[bit] > 0)
					sol = (sol + pw2[cnt0[bit] + cnt1[bit] - 1 + bit]) % kMod;
			}
		}

		for (int it : tree[node]) {
			if (!dsu.active[it])
				continue;

			int nxt = dsu.get_root(it);
			for (int bit = 0; bit < 31; ++bit) {
				if (vals[node] & (1 << bit)) {
					if (dsu.data[nxt].cnt1[bit] > 0)
						sol = (sol - 1LL * (pw2[dsu.data[nxt].cnt0[bit] + dsu.data[nxt].cnt1[bit] - 1] - 1) * pw2[bit]) % kMod;
					else
						sol = (sol - 1LL * (pw2[dsu.data[nxt].cnt0[bit]] - 1) * pw2[bit]) % kMod;
				}
				else {
					if (dsu.data[nxt].cnt1[bit] > 0)
						sol = (sol - pw2[dsu.data[nxt].cnt0[bit] + dsu.data[nxt].cnt1[bit] - 1 + bit]) % kMod;
				}
				if (sol < 0) sol += kMod;
			}
		}

		for (int it : tree[node]) {
			if (!dsu.active[it])
				continue;
			dsu.join(it, node);
		}
	}

	cout << sol << '\n';

	return 0;
}

//Trust me, I'm the Doctor!
