#include <vector>
#include <algorithm>
#include <numeric>  
#include <iostream>
#include <set>
#include <map>
#include <cassert>
#include <fstream>
using namespace std;

int sigma;
auto cmp = [](int x, int y){
    return (x + sigma - 1) % sigma < (y + sigma - 1) % sigma; };

using ll = long long;
constexpr ll mod = 1e9 + 7;

int tinf(int n, int sigma){
    return n < sigma ? n : (n + tinf(n/sigma, sigma))%sigma; }

vector<int> build_ad(vector<int> v, int sigma){
    vector<int> ad(v.size());
    adjacent_difference(begin(v), end(v), begin(ad), [sigma](int x, int y){
        return (x - y + sigma)%sigma; });
    ad.erase(begin(ad));
    return ad; }

int brut(vector<int> v, int sigma){
    for(int i = 0; ; ++i){
        for(int j = 0; j < v.size(); ++j)
            if(v[j] != tinf(i+j, sigma))
                goto out;
        return i;
        out: ; } }

bool has_split(vector<int> v, int sigma){
    auto ad = build_ad(v, sigma);
    return any_of(begin(ad), end(ad), [](int x){ return x != 1; }); }
    
int find_split(vector<int> v, int sigma){
    auto ad = build_ad(v, sigma);
    return find_if(begin(ad), end(ad), [](int x){ return x != 1; }) - begin(ad); }

int mex(vector<int> v){
    set<int> st;
    for(auto& x : v) st.insert(x);
    for(int i = 0; ; ++i, i %= sigma)
        if(st.find(i) == end(st))
            return i; }

int mdoublex(vector<int> v){
    map<int, int> mp;
    for(auto x : v) mp[x] ++;
    for(int i = 1; ; ++i, i %= sigma)
        if(mp[i] != 2)
            return i; }

/*

2 0 -> 4
1 0 -> 3 * 4 + 2 = 14
0 0 -> 3 * 14 + 2 = 44

*/

int *match(vector<int> v, int sigma){
    //for(auto& x : v) cout << x << ' ';
    //cout << endl;
    if(v.size() == 1) return new int (v[0]);
    if(v.size() == 2 && (v[0]+1)%sigma == v[1]){
        if(v[0] != sigma - 1) return new int(v[0]);
        else return new int(2 * sigma - 2); }
    auto ad = build_ad(v, sigma);

    for(int i = 0, j = 1, k = 2; k < v.size(); ++i, ++j, ++k)
        if(v[i] == v[j] && v[j] == v[k])
            return nullptr;

    if(has_split(v, sigma)){
        auto poz = find_if(begin(ad), end(ad), [](int x){ return x != 1; }) - begin(ad) + 1; 
        poz %= sigma;
        for(int i = 0; i < ad.size(); ++i)
            if((i-poz+1)%sigma != 0 && ad[i] != 1){
                return nullptr; }
            
        vector<int> v_;
        if(poz != 0) v_.push_back((v[poz-1] + 1) % sigma);
        while(poz < v.size()){
            v_.push_back(v[poz]);
            poz += sigma; }
        poz %= sigma;

        auto tmp = match(v_, sigma);

        return tmp ? new int((sigma - poz) % sigma + sigma * *tmp) : nullptr; }

    if(v.size() > 2 * sigma) return nullptr;
    if(v.size() > sigma){
        int which = (v.size() == 2 * sigma ? v[0] : min(v[0], mdoublex(v), cmp));
        int where = find(begin(v), end(v), which) - begin(v);
        return new int((sigma - where) % sigma + sigma * *match(vector<int> { which, which }, sigma )); }
    else{
        int which = (v.size() < sigma ? min(v[0], mex(v)) : v[0]);
        return new int((v[0] - which + sigma) % sigma + sigma * *match(vector<int> { which }, sigma)); } }

int main(){
    int t;
    ifstream f("pscfft.in");
    ofstream g("pscfft.out");
    f >> t;
    while(t--){
        int n;
        f >> n >> sigma;
        vector<int> v(n);
        for(auto& x : v) f >> x;
        auto tmp = match(v, sigma);
        if(sigma == 1) g << 0 << '\n';
        else if(tmp){
            g << *tmp << '\n';
            //cout << *tmp << ' ' << brut(v, sigma) << endl;
        }
        else g << -1 << '\n'; }
    return 0; }

